// Copyright 2002-2003 University of Guelph, All Rights Reserved
import java.io.*;
import java.util.*;

class Main {

  public static Problem problem;
  public static boolean quiet;
  public static PrintWriter solnOut;
  public static boolean useSolnFile;

  public static String EXE_NAME = "Superclass of program to run";
  
  public static void main(String[] Args) throws IOException, Exception {
	quiet = false;
	useSolnFile = false;

	if (Args.length < 1) {
	  System.err.println("Too few parameters.");
	  printUsageMessage();
	  System.exit(1);
	} else {
	  if ((Args.length == 2) || (Args.length == 3)) {
		if (!Args[0].equals("-q")) {
		  System.err.println("Incorrect command line.");
		  printUsageMessage();
		  System.exit(1);
		} else {
		  quiet = true;
		  Debug.DEBUG_LEVEL = Debug.DEBUG_OFF;
		}
	  }

	  if (Args.length == 3) {
		parseConstraintFile(Args[2]);
		solnOut = new PrintWriter(new FileWriter(Args[1]));		
		useSolnFile = true;
	  } else if (Args.length == 2) {
		parseConstraintFile(Args[1]);
	  } else {
		parseConstraintFile(Args[0]);
	  }
	}
  }

  public static void parseConstraintFile(String PathToConstraintFile) throws IOException, Exception {

	BufferedReader constraintReader = new BufferedReader(new FileReader(PathToConstraintFile));

	String currentLine = constraintReader.readLine();

	int problemType;
	int lineNumber = 0;

	if (currentLine == null) {
	  throw new IOException("Empty file");
	}
	lineNumber = 1;

	if (currentLine.substring(0, 1).equals("Z")) {
	  problemType = Problem.ZEBRA;
	} else if (currentLine.substring(0, 1).equals("S")) {
	  problemType = Problem.SHERLOCK;
	} else {
	  throw new Exception("First character of file must be either 'Z' for Zebra or 'S' for Sherlock.");
	}

	if (Debug.DEBUG_LEVEL >= Debug.DEBUG_QUIET) {
	  if (problemType == Problem.ZEBRA) {
		System.out.println("File is a Zebra problem.");
	  } else if (problemType == Problem.SHERLOCK) {
		System.out.println("File is a Sherlock problem.");
	  } else {
		System.out.println("File is an unknown problem.");
	  }
	}

	problem = new Problem(problemType);

	currentLine = constraintReader.readLine();
	lineNumber = 2;

	while (currentLine != null) {
	  if (!(currentLine.substring(0,1).equals("#"))) {
		// Only parse line if is not a comment
		StringTokenizer stringTokens = new StringTokenizer(currentLine, " ");
		if (stringTokens.countTokens() != 3) {
		  if (stringTokens.countTokens() != 4) {
			throw new Exception((String.valueOf(lineNumber)) + ": constraint must be of the form v1 relation v2 or v1 between/not-between v2 v3");
		  } else if (problemType != Problem.SHERLOCK) {			
			throw new Exception((String.valueOf(lineNumber)) + ": constraint must be of the form v1 relation v2.");
		  } else {
			String v1String = stringTokens.nextToken();
			String relationString = stringTokens.nextToken();
			String v2String = stringTokens.nextToken();
			String v3String = stringTokens.nextToken();

			int v1 = (new Integer(v1String)).intValue();
			int v2 = (new Integer(v2String)).intValue();
			int v3 = (new Integer(v3String)).intValue();

			try {
			  problem.newRelation(v1, relationString, v2, v3);
			} catch (Exception E) {
			  throw (new Exception(String.valueOf(lineNumber) + E.getMessage()));
			}
		  }
		} else {

		  String v1String = stringTokens.nextToken();
		  String relationString = stringTokens.nextToken();
		  String v2String = stringTokens.nextToken();
		  
		  int v1 = (new Integer(v1String)).intValue();
		  int v2 = (new Integer(v2String)).intValue();
		  
		  if (!relationString.equals("is")) {
			try {
			  problem.newRelation(v1, relationString, v2);
			} catch (Exception E) {
			  throw (new Exception(String.valueOf(lineNumber) + " " + E));
			}
		  } else {
			problem.givenVariable(v1, v2);
		  }
		}
	  }
	  
	  currentLine = constraintReader.readLine();
      lineNumber++;
	}
  }
  
  public static void printUsageMessage() {
	System.out.println("Usage: java " + EXE_NAME + " [-q [solution-file]] problem-file");
	System.out.println("");
	System.out.println("The optional -q specifies quiet mode (only print the number of checks)");
    System.out.println("problem-file defines whether the the problem is a Zebra or Sherlock,");
    System.out.println("and the constraints which define the problem.");
	System.out.println("The optional solution-file is used to output the solution when in quiet mode");
	System.out.println("See README for file format:");
    System.out.println("");
  }
}

