// Copyright 2002-2003 University of Guelph, All Rights Reserved
import java.util.*;
import java.io.*;

class MutateOnly {

  public static final int POPULATION_SIZE = 400;
  public static final int NUM_UNFIT = 125;
  public static final int POOR_FIT_BUT_PARENT = 40;

  public TreeSet population;
  public Random mutateRandom;
  public Problem problem;
  public static Vector rows;
  public HashSet parents;

  public MutateOnly(Problem problemObject) {
	problem = problemObject;
	initPopulation();
  }

  public void findSolution() {   
	Mutate.numCases = 0;
    evolveSolution();
  }

  public void evolveSolution() {	
	mutateRandom = new Random();
	fitnessSort();

	while (bestNotSolution()) {
	  select();
	  reproduce();
	  fitnessSort();

	  if (Debug.DEBUG_LEVEL >= Debug.DEBUG_QUIET) {
		System.out.println("Tested " + String.valueOf(Mutate.numCases) + 
						 " possible solutions; Top fitness: " +
						 String.valueOf(((SimpleChromosone)(population.first())).getFitness()));	  
	  }
	 
	  if (Debug.DEBUG_LEVEL >= Debug.DEBUG_MEDIUM) {
		Iterator popIterator = population.iterator();
		int i = 0;
		SimpleChromosone individual;
		while (popIterator.hasNext() && (i < 5)) {
		  individual = ((SimpleChromosone)(popIterator.next()));
		  System.out.println(individual);
		  if (Debug.DEBUG_LEVEL >= Debug.DEBUG_HIGH) {
			System.out.println(individual.getSolutionString());
		  }
		  i++;
		}
		//		for (int i = popArray.length - 5; i < popArray.length; i++) {
		//		  System.out.println(((SimpleChromosone)(popArray[i])));
		//		}		
	  }
	  problem.solution = ((SimpleChromosone)(population.first())).getSolution();
	}
  }

  public boolean bestNotSolution() {
	boolean isSolution = (((SimpleChromosone)(population.first())).isSolution());

	if (!isSolution) {
	  if (Debug.DEBUG_LEVEL >= Debug.DEBUG_HIGH) {
		System.out.print(" " + String.valueOf(problem.checkConflicts()) + " ");
	  }
	}
	return !isSolution;
  }

  public void fitnessSort() {
	//	Arrays.sort(population);
  }

  public int getParent(double chanceRepro[]) {
	double randParent = mutateRandom.nextDouble();

	int i = 0;
	double maxChance = chanceRepro[i];
	
	while(randParent > maxChance) {
	  i++;
	  maxChance += chanceRepro[i];
	}
	return i;
  }

  public int getPoorParent(double chanceRepro[]) {
	double randParent = mutateRandom.nextDouble();

	int i = 0;
	double maxChance = chanceRepro[i];
	
	while(randParent > maxChance) {
	  i++;
	  maxChance += chanceRepro[i];
	}
	return i;
  }

  public void select() {
	
	Object popArray[] = population.toArray();
	double bottomFitness = ((SimpleChromosone)(popArray[POPULATION_SIZE - NUM_UNFIT - 1])).getFitness();
	double chanceRepro[] = new double[POPULATION_SIZE - NUM_UNFIT];
	double timesBetter[] = new double[POPULATION_SIZE - NUM_UNFIT];
	double totalTimesBetter = 0;

	for (int i = 0; i < POPULATION_SIZE - NUM_UNFIT; i++) {
	  timesBetter[i] = (10000 - ((SimpleChromosone)(popArray[i])).getFitness()) / (10000 - bottomFitness);
	  totalTimesBetter += timesBetter[i];
	}
	double x = 1 / totalTimesBetter;

	for (int i = 0; i < POPULATION_SIZE - NUM_UNFIT; i++) {
	  chanceRepro[i] = timesBetter[i] * x;
	}

	int parent;
	parents = new HashSet();

	for (int i = 0; i < (NUM_UNFIT - POOR_FIT_BUT_PARENT + 1); i++) {
	  parent = getParent(chanceRepro);	
	  parents.add(((SimpleChromosone)(popArray[parent])).clone());	  
	}

	for (int i = 0; i < POPULATION_SIZE - NUM_UNFIT; i++) {
	  chanceRepro[i] = 1 - chanceRepro[i];
	}

	for (int i = 0; i < (POOR_FIT_BUT_PARENT + 1); i++) {
	  parent = getParent(chanceRepro);
	  parents.add(((SimpleChromosone)(popArray[parent])).clone());
	}
  }

  public void reproduce() {
	SimpleChromosone individual;
	Iterator parentIterator = parents.iterator();
	SimpleChromosone parent;
	HashSet unfitMembers = new HashSet();
	Object popArray[] = population.toArray();

	for (int i = POPULATION_SIZE - NUM_UNFIT; i < POPULATION_SIZE; i++) {
	  unfitMembers.add(popArray[i]);
	}
	
	while (parentIterator.hasNext()) {
	  Mutate.numCases++;
	  individual = (SimpleChromosone)(parentIterator.next());	   
	  do {
		individual.mutate(mutateRandom, problem.getMaxVars());
	  } while (population.contains(individual));
	  population.add(individual);	  
	}
	Iterator unfitIterator = unfitMembers.iterator();
	while (unfitIterator.hasNext()) {
	  population.remove(unfitIterator.next());
	}
  }

  public void initPopulation() {
	int maxDomain = problem.getMaxDomain();
	Vector varPermutations;
	TreeSet topDomain = new TreeSet();
	Vector varSolution;
	Random permRandom;

	for (int i = 1; i <= maxDomain; i++) {
	  topDomain.add(new Integer(i));
	}

	varPermutations = doPermute(topDomain);
	rows = varPermutations;

	population = new TreeSet();

	permRandom = new Random();
	int permChoice;
	int solution[];
	SimpleChromosone individual;

	solution = new int[problem.getMaxVars() + 1];
	
	for (int i = 0; i < POPULATION_SIZE; i++) {
	  Mutate.numCases++;
	  for (int j = 0; j < problem.getMaxDomain(); j++) {
		permChoice = permRandom.nextInt(varPermutations.size());
		setSolution(varPermutations.elementAt(permChoice), j * problem.getMaxDomain() + 1, solution);
	  }		
	  individual = new SimpleChromosone(solution, problem);
	  while (population.contains(individual)) {
		for (int j = 0; j < problem.getMaxDomain(); j++) {
		  permChoice = permRandom.nextInt(varPermutations.size());
		  setSolution(varPermutations.elementAt(permChoice), j * problem.getMaxDomain() + 1, solution);
		}
		individual = new SimpleChromosone(solution, problem);
	  }	 
	  population.add(individual);			
	}
  }

  public static void setSolution(Object permVector, int varStart, int solution[]) {
	Vector permutes = (Vector)(permVector);
	for (int o = 0; o < permutes.size(); o++) {
	  solution[varStart + o] = 
		((Integer)(permutes.elementAt(o))).intValue();
	}
  }

  public static Vector doPermute(TreeSet curDomain) {
	Vector permutations = new Vector();
	Vector integers;

	if (curDomain.size() == 1) {
	  integers = new Vector();
	  integers.addElement((Integer)(curDomain.first()));
	  permutations.addElement(integers);
	} else {	  
	  Iterator current = curDomain.iterator();	 
	  while (current.hasNext()) {
		TreeSet newDomain = (TreeSet)(curDomain.clone());
		Integer curInt = (Integer)(current.next());
		newDomain.remove(curInt);

		Vector endCombos = doPermute(newDomain);
		for (int i = 0; i < endCombos.size(); i++) {
		  integers = new Vector();
		  integers.addElement(curInt);
		  Vector endInts = (Vector)(endCombos.elementAt(i));

		  for (int j = 0; j < endInts.size(); j++) {
			integers.addElement((Integer)(endInts.elementAt(j)));
		  }
		  permutations.addElement(integers);
		}		
	  }	  
	}

	if (Debug.DEBUG_LEVEL >= Debug.DEBUG_VERBOSE) {
	  for (int i = 0; i < permutations.size(); i++) {
		Vector ints = (Vector)(permutations.elementAt(i));
		System.out.print(String.valueOf(i) + ": ");
		for (int j = 0; j < ints.size(); j++) {
		  System.out.print(String.valueOf((Integer)(ints.elementAt(j)) + " "));
		}
		System.out.println("");
	  }
	}
	return permutations;
  }
}

